VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Mutex"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Declare Function CreateMutex Lib "kernel32" Alias "CreateMutexA" (lpMutexAttributes As SECURITY_ATTRIBUTES, ByVal bInitialOwner As Long, ByVal lpName As Long) As Long
Private Declare Function OpenMutex Lib "kernel32" Alias "OpenMutexA" (ByVal dwDesiredAccess As Long, ByVal bInheritHandle As Long, ByVal lpName As Long) As Long

Private Declare Function ReleaseMutex Lib "kernel32" (ByVal hMutex As Long) As Long
Private Declare Function WaitForSingleObject Lib "kernel32" (ByVal hHandle As Long, ByVal dwMilliseconds As Long) As Long

Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

Private Type SECURITY_ATTRIBUTES
    nLength As Long
    lpSecurityDescriptor As Long
    bInheritHandle As Long
End Type

Private Const INFINITE = &HFFFFFFFF

Private Const STATUS_WAIT_0 = &H0
Private Const STATUS_ABANDONED_WAIT_0 = &H80

Private Const WAIT_OBJECT_0 = (STATUS_WAIT_0 + 0)
Private Const WAIT_FAILED = &HFFFFFFFF
Private Const WAIT_ABANDONED = (STATUS_ABANDONED_WAIT_0 + 0)
Private Const WAIT_TIMEOUT = 258&

Private Const SYNCHRONIZE = &H100000

Dim hMutex As Long
Dim MutexLockCount As Long

Public Event Error(ByVal MutexError As Long, ByVal Description As String)
Public Property Get MutexHandle() As Long
MutexHandle = hMutex
End Property
Public Property Get isLocked() As Boolean
isLocked = False
If MutexLockCount > 0 Then isLocked = True
End Property
Private Sub Class_Initialize()
Dim tMutex As Long
Dim tSA As SECURITY_ATTRIBUTES
Dim NullString As String

tSA.bInheritHandle = 0
tSA.lpSecurityDescriptor = 0
tSA.nLength = Len(tSA)

'Erstellung einen anonymen Mutex
tMutex = CreateMutex(tSA, ByVal 0, ByVal 0)
If tMutex = 0 Then
    RaiseEvent Error(1, "Mutex konnte nicht initialisiert werden.")
Else
    hMutex = tMutex
End If
End Sub
Public Sub InitNamedMutex(ByVal Name As String, Optional ByVal Signaled As Boolean = False)
Dim tMutex As Long
Dim tSA As SECURITY_ATTRIBUTES
Dim NullString As String

'Wenn Signaled gleich True ist, wird der Mutex automatisch
'in Besitz genommen nachdem der Mutex erstellt wurde

tSA.bInheritHandle = 0
tSA.lpSecurityDescriptor = 0
tSA.nLength = Len(tSA)

If Not StrComp(Right(Name, 1), Chr(0), vbBinaryCompare) = 0 Then Name = Name & Chr(0)

tMutex = CreateMutex(tSA, Signaled, ByVal StrPtr(Name))
If tMutex = 0 Then
    RaiseEvent Error(1, "Mutex konnte nicht initialisiert werden.")
Else
    Call CloseHandle(hMutex)
    hMutex = tMutex
    MutexLockCount = 0
    If Signaled Then MutexLockCount = 1
End If
End Sub
Public Sub OpenExistingMutex(ByVal Name As String)
Dim retval As Long
If Not StrComp(Right(Name, 1), Chr(0), vbBinaryCompare) = 0 Then Name = Name & Chr(0)
retval = OpenMutex(SYNCHRONIZE, 0, ByVal StrPtr(Name))

If retval = 0 Then
    RaiseEvent Error(7, "Mutex mit dem angegebenem Namen existiert nicht.")
Else
    CloseHandle (hMutex)
    hMutex = retval
    MutexLockCount = 0
End If
End Sub
Private Sub Class_Terminate()
'Freigabe des Handles auf den Mutex
Call CloseHandle(hMutex)
End Sub
Public Sub MutexLock()
'Versucht den Mutex zu bekommen
'Das Timeout wird quasi nicht beachtet
Call MutexLockMs(INFINITE)
End Sub
Public Sub MutexLockMs(ByVal Milliseconds As Long)
Dim retval As Long
If Milliseconds < -1 Then Milliseconds = 0
retval = WaitForSingleObject(hMutex, Milliseconds)
If retval = WAIT_FAILED Then
    'Dieser Fehler tritt auf, wenn bei der Verwendung der API-Funktion
    'ein Fehler gemacht wurde ....
    RaiseEvent Error(3, "Bei der Verwendung von WaitForSingleObject ist ein Fehler aufgetreten.")
ElseIf retval = WAIT_ABANDONED Then
    'Falls der Thread, welcher den Mutex gerade besa, beendet wurde
    'ohne das er den Mutex per ReleaseMutex wieder freigegeben hat,
    'so ist der Status des Mutex nicht definiert
    RaiseEvent Error(4, "Ein anderer Thread hat bei der Arbeit am Mutex einen Fehler gemacht. Der Status des Mutex ist nicht definiert.")
ElseIf retval = WAIT_TIMEOUT Then
    'Das angegebene Timeout wurde berschritten
    RaiseEvent Error(5, "Die Funktion WaitForSingleObject konnte den Mutex nicht in Besitz nehmen, da das Timeout berschritten wurde.")
ElseIf retval = WAIT_OBJECT_0 Then
    'Der Mutex konnte erfolgreich in Besitz genommen werden
    MutexLockCount = MutexLockCount + 1
Else
    'Ein anderen Fehler
    RaiseEvent Error(6, "Der Rckgabewert von WaitForSingleObject wurde nicht behandelt.")
End If
End Sub
Public Sub MutexUnlock()
'Gibt den Mutex wieder aus der Hand
Dim retval As Long
retval = ReleaseMutex(hMutex)
If retval = 0 Then
    RaiseEvent Error(2, "Mutex konnte nicht freigegeben werden, da er keinen Lock auf den Mutex hatte.")
Else
    MutexLockCount = MutexLockCount - 1
End If
End Sub
